import logging
import logging.handlers
import pkg_resources
import socket


class ContextFilter(logging.Filter):
    hostname = socket.gethostname()
    version = pkg_resources.get_distribution('rpi_metar').version

    def filter(self, record):
        record.hostname = ContextFilter.hostname
        record.version = ContextFilter.version
        return True


ctx_filter = ContextFilter()


def init_logger():

    log = logging.getLogger(__name__)

    log.setLevel(logging.DEBUG)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(version)s - %(threadName)s - %(message)s')
    handler = logging.handlers.SysLogHandler(address='/dev/log')
    handler.setFormatter(formatter)
    handler.addFilter(ctx_filter)
    log.addHandler(handler)


init_logger()



"""Weather parsing utilities."""
import logging
import re
from enum import Enum
from fractions import Fraction
from rpi_metar.leds import GREEN, RED, BLUE, MAGENTA, YELLOW, BLACK, ORANGE

log = logging.getLogger(__name__)


class FlightCategory(Enum):
    VFR = GREEN
    IFR = RED
    MVFR = BLUE
    LIFR = MAGENTA
    UNKNOWN = YELLOW
    OFF = BLACK
    MISSING = ORANGE


def get_conditions(metar_info):
    """Returns the visibility, ceiling, wind speed, and gusts for a given airport from some metar info."""
    log.debug(metar_info)
    visibility = ceiling = None
    speed = gust = 0
    # Visibility

    # Match metric visibility and convert to SM
    match = re.search(r'(?P<CAVOK>CAVOK)|(\s(?P<visibility>\d{4}|\/{4})\s)', metar_info)
    if match:
        if match.group('visibility'):
            try:
                visibility = float(match.group('visibility')) / 1609
            except ValueError:
                visibility = None
        if match.group('CAVOK'):
            visibility = 10

    # Match SM Visibility
    # We may have fractions, e.g. 1/8SM or 1 1/2SM
    # Or it will be whole numbers, e.g. 2SM
    # There's also variable wind speeds, followed by vis, e.g. 300V360 1/2SM
    match = re.search(r'(?P<visibility>\b(?:\d+\s+)?\d+(?:/\d)?)SM', metar_info)
    if match:
        visibility = match.group('visibility')
        try:
            visibility = float(sum(Fraction(s) for s in visibility.split()))
        except ZeroDivisionError:
            visibility = None
    # Ceiling
    match = re.search(r'(VV|BKN|OVC)(?P<ceiling>\d{3})', metar_info)
    if match:
        ceiling = int(match.group('ceiling')) * 100  # It is reported in hundreds of feet
    # Wind info
    match = re.search(r'\b\d{3}(?P<speed>\d{2,3})G?(?P<gust>\d{2,3})?KT', metar_info)
    if match:
        speed = int(match.group('speed'))
        gust = int(match.group('gust')) if match.group('gust') else 0
    return (visibility, ceiling, speed, gust)


def get_flight_category(visibility, ceiling):
    """Converts weather conditions into a category."""
    log.debug('Finding category for %s, %s', visibility, ceiling)
    if visibility is None and ceiling is None:
        return FlightCategory.UNKNOWN

    # Unlimited ceiling
    if visibility and ceiling is None:
        ceiling = 10000

    # http://www.faraim.org/aim/aim-4-03-14-446.html
    try:
        if visibility < 1 or ceiling < 500:
            return FlightCategory.LIFR
        elif 1 <= visibility < 3 or 500 <= ceiling < 1000:
            return FlightCategory.IFR
        elif 3 <= visibility <= 5 or 1000 <= ceiling <= 3000:
            return FlightCategory.MVFR
        elif visibility > 5 and ceiling > 3000:
            return FlightCategory.VFR
    except (TypeError, ValueError):
        log.exception('Failed to get flight category from {vis}, {ceil}'.format(
            vis=visibility,
            ceil=ceiling
        ))



import csv
import logging
import re
import requests
import time

from pkg_resources import resource_filename
from retrying import retry
from xmltodict import parse as parsexml

log = logging.getLogger(__name__)


def chunks(l, n):
    """Yield successive n-sized chunks from l."""
    for i in range(0, len(l), n):
        yield l[i:i + n]


class METARSource:

    @retry(wait_exponential_multiplier=1000,
           wait_exponential_max=10000,
           stop_max_attempt_number=10)
    def _query(self):
        """Queries the NOAA METAR service."""
        log.info(self.url)
        try:
            response = requests.get(self.url, timeout=10.0)
            response.raise_for_status()
        except:  # noqa
            log.exception('Metar query failure.')
            raise
        return response


class NOAA(METARSource):

    URL = (
        'https://{subdomain}.aviationweather.gov/adds/dataserver_current/httpparam'
        '?dataSource=metars'
        '&requestType=retrieve'
        '&format=xml'
        '&hoursBeforeNow=2'
        '&mostRecentForEachStation=true'
        '&stationString={airport_codes}'
    )

    def __init__(self, airport_codes, subdomain='www'):
        self.airport_codes = airport_codes
        self.subdomain = subdomain

    def get_metar_info(self):
        """Queries the NOAA METAR service."""
        metars = {}

        # NOAA can only handle so much at once, so split into chunks.
        # Even though we can issue larger chunk sizes, sometimes data is missing from the returned
        # results. Smaller chunks seem to help...
        for chunk in chunks(self.airport_codes, 250):
            self.url = self.URL.format(airport_codes=','.join(chunk), subdomain=self.subdomain)
            response = self._query()
            try:
                response = parsexml(response.text)['response']['data']['METAR']
                if not isinstance(response, list):
                    response = [response]
            except:  # noqa
                log.exception('Metar response is invalid.')
                raise
            finally:
                # ...but with more requests, we should be nice and wait a bit before the next
                time.sleep(1.0)

            for m in response:
                metars[m['station_id'].upper()] = m

        return metars


class NOAABackup(NOAA):

    def __init__(self, airport_codes):
        super(NOAABackup, self).__init__(airport_codes, subdomain='bcaws')


class SkyVector(METARSource):

    URL = (
        'https://skyvector.com/api/dLayer'
        '?ll1={lat1},{lon1}'  # lower left
        '&ll2={lat2},{lon2}'  # upper right
        '&layers=metar'
    )

    def _find_coordinates(self):
        data = {}
        file_name = resource_filename('rpi_metar', 'data/us-airports.csv')
        with open(file_name, newline='') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                airport_code, lat, lon = row
                if airport_code in self.airport_codes:
                    data[airport_code] = (lat, lon)

        self.data = data

        lat1 = min((float(lat) for lat, _ in data.values()))
        lon1 = min((float(lon) for _, lon in data.values()))
        lat2 = max((float(lat) for lat, _ in data.values()))
        lon2 = max((float(lon) for _, lon in data.values()))

        # skyvector either isn't inclusive, or our data doesn't match theirs. Regardless, we
        # must expand the search area slightly.
        lat1, lon1 = map(lambda x: x - 0.5, [lat1, lon1])
        lat2, lon2 = map(lambda x: x + 0.5, [lat2, lon2])

        self.url = SkyVector.URL.format(lat1=lat1, lon1=lon1, lat2=lat2, lon2=lon2)

    def __init__(self, airport_codes):
        # Set lat / long info for the request...
        self.airport_codes = [code.upper() for code in airport_codes]
        self._find_coordinates()

    def get_metar_info(self):
        response = self._query()
        try:
            data = response.json()['weather']
        except:  # noqa
            log.exception('Metar response is invalid.')
            raise

        """Sample response:
        [{'a': '01h 02m ago',
         'd': '2018-08-22 18:56:00',
         'i': '0VFR.png',
         'lat': '40.4518278',
         'lon': '-105.0113361',
         'm': 'KFNL 221856Z AUTO VRB03KT 6SM HZ CLR 23/14 A3025 RMK AO2 SLP194 T02280139 PNO $',
         'n': 'FT COLLINS/LOVEL',
         's': 'KFNL',
         't': None}, ... ]
        """

        # Make the return match the format of the other sources.
        metars = {}
        for item in data:
            if item['s'] in self.airport_codes:
                metars[item['s'].upper()] = {'raw_text': item['m']}

        return metars


class BOM(METARSource):
    """Queries the BOM website service."""

    URL = 'http://www.bom.gov.au/aviation/php/process.php'

    def __init__(self, airport_codes):
        self.airport_codes = ','.join(airport_codes)

    def get_metar_info(self):

        payload = {
            'keyword': self.airport_codes,
            'type': 'search',
            'page': 'TAF',
        }

        r = requests.post(self.URL, data=payload)

        matches = re.finditer(r'(?:METAR |SPECI )(?P<METAR>(?P<CODE>\w{4}).*?)(?:<br />|<h3>)', r.text)

        metars = {}
        for match in matches:
            info = match.groupdict()
            metars[info['CODE'].upper()] = {'raw_text': info['METAR']}

        return metars



#!/usr/bin/env python
from rpi_metar import sources, core
from rpi_metar.leds import GREEN, BLACK
from rpi_ws281x import PixelStrip
import configparser


def main():
    config = configparser.ConfigParser()
    config['settings'] = {'brightness': 75}
    cfg_file = '/etc/rpi_metar.conf'

    leds = PixelStrip(num=2000, pin=18, gamma=core.GAMMA, brightness=128)
    leds.begin()

    # If there's an existing config file, see if we want to continue where we left off or just
    # overwrite it.
    i = 0
    airports = {}
    try:
        config.read([cfg_file])
    except:
        pass
    else:
        prompt = None
        while prompt not in ['c', 'o']:
            prompt = input('cfg file exists.  [c]ontinue or [o]verwrite')
        if prompt == 'c':
            for code in config.options('airports'):
                index = config.getint('airports', code)
                airports[code.upper()] = index
            i = max(airports.values()) + 1

    code = None
    while code != 'q':
        core.set_all(leds, BLACK)
        leds.setPixelColor(i, GREEN)
        leds.show()

        code = input('Code:   [s]kip or [q]uit ').casefold()
        if code == 'q':
            break
        if code == 's':
            i += 1
            continue

        if len(code) == 3:
            code = 'k' + code
        code = code.upper()
        noaa = sources.NOAA([code])
        try:
            noaa.get_metar_info()
        except:
            prompt = None
            while prompt not in ['k', 'r']:
                prompt = input('{code} is invalid.  [k]eep or [r]etry '.format(code=code))
            if prompt == 'r':
                continue

        if code in airports:
            prompt = None
            while prompt not in ['k', 'r']:
                prompt = input('{code} has already been set.  [k]eep or [r]etry '.format(code=code))
            if prompt == 'r':
                continue

        airports[code] = i
        i += 1

    core.set_all(leds, BLACK)
    config['airports'] = airports
    with open(cfg_file, 'w') as f:
        config.write(f)
    print('Wrote {file}'.format(file=cfg_file))



import logging
import re
from queue import Queue
from rpi_metar import wx

MAX_WIND_SPEED_KTS = 30  # When it's too windy, in knots.

log = logging.getLogger(__name__)


LED_QUEUE = Queue()


class Airport(object):

    def __init__(self, code, led_index, max_wind_speed_kts=MAX_WIND_SPEED_KTS, unknown_off=True):
        self.code = code.upper()
        self.index = led_index
        self.visibility = None
        self.ceiling = None
        self.raw = None
        self.thunderstorms = False
        self.wind_speed = 0
        self.wind_gusts = 0
        self.max_wind_speed = max_wind_speed_kts
        self._category = wx.FlightCategory.UNKNOWN
        self._unknown_count = 0
        self._unknown_off = unknown_off

    def __repr__(self):
        return '<{code} @ {index}: {raw} -> {cat}>'.format(
            code=self.code,
            index=self.index,
            raw=self.raw,
            cat=self.category.name
        )

    def reset(self):
        self.visibility = None
        self.ceiling = None
        self.raw = None
        self.thunderstorms = False
        self.wind_speed = 0
        self.wind_gusts = 0

    @property
    def windy(self):
        return (self.wind_speed > self.max_wind_speed or self.wind_gusts > self.max_wind_speed) and self.category != wx.FlightCategory.OFF

    @property
    def category(self):
        return self._category

    @category.setter
    def category(self, cat):
        if cat is None:
            cat = wx.FlightCategory.UNKNOWN

        if cat == wx.FlightCategory.UNKNOWN:
            self._unknown_count += 1
            if self._unknown_count >= 3:
                if self._unknown_off:
                    cat = wx.FlightCategory.OFF
                else:
                    cat = wx.FlightCategory.MISSING
        else:
            self._unknown_count = 0

        if self._category != cat:
            log.info('Changing {self} to {cat}'.format(self=self, cat=cat))
            self._category = cat
            log.info('Setting category, putting {} onto queue.'.format(self.code))
            LED_QUEUE.put(self.code)

    def process_metar(self, metars):
        # Make sure previous iterations are cleared out
        self.reset()

        try:
            metar = metars[self.code]
            log.debug(metar)
            self.raw = metar['raw_text']
        except KeyError:
            log.exception('{} has no data.'.format(self.code))
            self.category = wx.FlightCategory.UNKNOWN
            return

        # Thunderstorms
        self.thunderstorms = bool(re.search(r'\w{4,5}.*(TS).*?(?=RMK)', metar['raw_text']) and self.category != wx.FlightCategory.OFF)

        # Wind info
        try:
            self.wind_speed = int(metar['wind_speed_kt'])
        except KeyError:
            pass

        try:
            self.wind_gusts = int(metar['wind_gust_kt'])
        except KeyError:
            pass

        # Flight categories. First automatic, then manual parsing.
        try:
            if metar['flight_category'] is None:
                log.error('flight category is missing: {}', metar)
            self.category = wx.FlightCategory[metar['flight_category']]
        except KeyError:
            log.info('%s does not have flight category field, falling back to raw text parsing.', self.code)
            self.visibility, self.ceiling, self.wind_speed, self.wind_gusts = wx.get_conditions(metar['raw_text'])
            self.category = wx.get_flight_category(self.visibility, self.ceiling)


class Legend(object):

    def __init__(self, name, index, category):
        self.name = name
        self.index = index
        self.category = category

    def __repr__(self):
        return '<LEGEND @ {index}: {name}>'.format(
            index=self.index,
            name=self.name
        )



from RPi import GPIO
import logging

log = logging.getLogger(__name__)


# The two pins that the encoder uses (BCM numbering).
GPIO_A = 23
GPIO_B = 25


class RotaryEncoder:

    def __init__(self, callback, gpio_a=GPIO_A, gpio_b=GPIO_B):
        self.last_gpio = None
        self.gpio_a = gpio_a
        self.gpio_b = gpio_b
        self.callback = callback

        self.level_a = 0
        self.level_b = 0

        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.gpio_a, GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.setup(self.gpio_b, GPIO.IN, pull_up_down=GPIO.PUD_UP)

        GPIO.add_event_detect(self.gpio_a, GPIO.BOTH, self._callback)
        GPIO.add_event_detect(self.gpio_b, GPIO.BOTH, self._callback)

    def destroy(self):
        GPIO.remove_event_detect(self.gpio_a)
        GPIO.remove_event_detect(self.gpio_b)
        GPIO.cleanup()

    def reset(self):
        self.last_gpio = None
        self.level_a = 0
        self.level_b = 0

    def _callback(self, channel):
        level = GPIO.input(channel)
        log.debug('{channel} = {level}'.format(channel=channel, level=level))
        if channel == self.gpio_a:
            self.level_a = level
        else:
            self.level_b = level

        if level != 1:
            return

        # When both inputs are at 1, we'll fire a callback. If A was the most recent pin set high,
        # it'll be forward, and if B was the most recent pin set high, it'll be reverse.
        if channel != self.last_gpio:  # debounce
            self.last_gpio = channel
            log.debug('set last_gpio to {channel}'.format(channel=channel))
            if channel == self.gpio_a and self.level_b == 1:
                log.debug('A is set and B was already set, callback(1)')
                self.callback(1)
                self.reset()
            elif channel == self.gpio_b and self.level_a == 1:
                log.debug('B is set and A was already set, callback(-1)')
                self.callback(-1)
                self.reset()



import logging

from crontab import CronTab

log = logging.getLogger(__name__)


def set_upgrade_schedule():
    """Makes sure the code tries to update itself periodically."""
    cron = CronTab(user='root')

    cmd = '/opt/rpi_metar/bin/upgrade.sh'
    changes_made = False

    # The code should check daily, and at every reboot
    daily_found = False
    reboot_found = False

    # Search for daily and reboot checks, removing anything else
    for job in cron.find_command(cmd):
        freq = job.frequency()
        if freq == 365:
            daily_found = True
        elif freq == 525600:
            reboot_found = True
        else:
            cron.remove(job)
            changes_made = True

    if not daily_found:
        job = cron.new(command=cmd)
        job.every(1).days()
        changes_made = True
        log.info('Adding daily update to cron.')
    if not reboot_found:
        job = cron.new(command=cmd)
        job.every_reboot()
        changes_made = True
        log.info('Adding reboot update to cron.')

    if changes_made:
        cron.write()
        log.info('Updated crontab.')



#!/usr/bin/env python
import requests
import logging
import logging.handlers
import os
import socket
import signal
import sys
import time
import threading
from configparser import ConfigParser
from rpi_ws281x import PixelStrip
from rpi_metar import cron, sources, encoder
from rpi_metar.leds import BLACK, YELLOW, WHITE, GAMMA
from rpi_metar.airports import Airport, LED_QUEUE, MAX_WIND_SPEED_KTS, Legend
from rpi_metar.wx import FlightCategory
from queue import Queue


log = logging.getLogger(__name__)


METAR_REFRESH_RATE = 5 * 60  # How often METAR data should be fetched, in seconds
WIND_DISPLAY_RATE = 5  # How often to show that it's windy, in seconds
LIGHTNING_STRIKE_RATE = 5  # How regularly should lightning strike, in seconds

FAILURE_THRESHOLD = 3  # How many times do we not get data before we reboot

ENCODER_QUEUE = Queue()
METAR_QUEUE = Queue()
ENCODER_EVENT = threading.Event()
METAR_EVENT = threading.Event()

# A collection of the airports we'll ultimately be tracking.
AIRPORTS = {}


def is_internet_up():
    try:
        response = requests.get('http://google.com', timeout=10.0)
        response.raise_for_status()
    except:  # noqa
        return False
    return True


def fetch_metars(queue, cfg):
    """Fetches new METAR information periodically."""
    failure_count = 0

    # Load the desired data sources from the user configuration.
    srcs = cfg.get('settings', 'sources', fallback='NOAA,NOAABackup,SkyVector').split(',')
    srcs = [getattr(sources, src.strip()) for src in srcs]

    while True:

        metars = {}
        airport_codes = set(AIRPORTS.keys())
        for source in srcs:
            try:
                data_source = source(list(airport_codes))
            except:  # noqa
                log.exception('Unable to create data source.')
                continue

            try:
                info = data_source.get_metar_info()
                log.info('Retrieved: %s', info)
                metars.update(info)
                failure_count = 0
            except:  # noqa
                log.exception('Failed to retrieve metar info.')

            # We have retrieved METAR info, but did we get responses for all stations? If we did
            # not, let's request those missing stations from the other sources. Perhaps they have
            # the info!
            airport_codes = airport_codes - set(metars.keys())
            if not airport_codes:
                # Nothing else needs to be retrieved
                break

        # We have exhausted the list of sources.
        if not metars:
            metars = None

            # Some of the raspberry pis lose their wifi after some time and fail to automatically
            # reconnect. This is a workaround for that case. If we've failed a lot, just reboot.
            # We do need to make sure we're not rebooting too soon (as would be the case for
            # initial setup).
            failure_count += 1

            # If other web services are available, it's just the NOAA site having problems so we
            # don't need to reboot.
            if failure_count >= FAILURE_THRESHOLD and not is_internet_up():
                log.warning('Internet is not up, rebooting.')
                os.system('reboot')

        queue.put(metars)
        time.sleep(cfg.getint('settings', 'metar_refresh_rate', fallback=METAR_REFRESH_RATE))


def process_metars(queue, leds):
    """Converts METAR info info Flight Categories and updates the LEDs."""

    airports = AIRPORTS.values()

    # When the system first starts up, waiting for all of the LEDs to fade into their correct
    # colors can take a very long time. To mitigate this, we'll just slam the colors into place
    # if this is the first time this thread is executing.
    first = True

    while True:

        try:
            metars = queue.get()
            if metars is None:
                for airport in airports:
                    airport.category = FlightCategory.UNKNOWN
                continue

            for airport in airports:
                airport.process_metar(metars)

                if first:
                    leds.setPixelColor(airport.index, airport.category.value)

            if first:
                first = False
                leds.show()

            # Let the weather checkers know the info is refreshed
            METAR_EVENT.set()

            log.info(sorted(AIRPORTS.values(), key=lambda x: x.index))

        except:  # noqa
            log.exception('metar processor error')


def render_leds(queue, leds, cfg):
    """Updates the LED strand when something pops onto the queue."""
    while True:
        log.info('waiting for queue.')
        airport_code = queue.get()
        log.info('got {}'.format(airport_code))
        airport = AIRPORTS[airport_code.upper()]
        # This is our target color.
        color = airport.category.value

        if not cfg.getboolean('settings', 'do_fade', fallback=True):
            leds.setPixelColor(airport.index, color)
            leds.show()
            continue

        # Let's try to fade to our desired color
        start_color = leds.getPixelColor(airport.index)
        start_g = start_color >> 16 & 0xff
        start_r = start_color >> 8 & 0xff
        start_b = start_color & 0xff

        end_g = color >> 16 & 0xff
        end_r = color >> 8 & 0xff
        end_b = color & 0xff

        with leds.lock:  # Don't let lightning or wind interrupt us.
            while((start_r != end_r) or (start_g != end_g) or (start_b != end_b)):
                if start_r < end_r:
                    start_r += 1
                elif start_r > end_r:
                    start_r -= 1
                if start_g < end_g:
                    start_g += 1
                elif start_g > end_g:
                    start_g -= 1
                if start_b < end_b:
                    start_b += 1
                elif start_b > end_b:
                    start_b -= 1

                leds.setPixelColorRGB(airport.index, start_g, start_r, start_b)
                leds.show()


def lightning(leds, event, cfg):
    """Briefly changes LEDs to white, indicating lightning in the area."""
    airports = AIRPORTS.values()
    strike_duration = cfg.getfloat('settings', 'lightning_duration', fallback=1.0)
    legend = cfg.getint('legend', 'lightning', fallback=None)
    legend = [Legend('LIGHTNING', legend, FlightCategory.OFF)] if legend else []
    while True:
        # Which airports currently are experiencing thunderstorms
        ts_airports = [airport for airport in airports if airport.thunderstorms] + legend
        log.debug("LIGHTNING @: {}".format(ts_airports))
        if ts_airports:
            with leds.lock:
                for airport in ts_airports:
                    leds.setPixelColor(airport.index, WHITE)
                leds.show()
                time.sleep(strike_duration)

                for airport in ts_airports:
                    leds.setPixelColor(airport.index, airport.category.value)
                leds.show()
            time.sleep(LIGHTNING_STRIKE_RATE - strike_duration)
        else:
            # Sleep until the next metar refresh...
            event.wait(cfg.getint('settings', 'metar_refresh_rate', fallback=METAR_REFRESH_RATE))
            event.clear()


def wind(leds, event, cfg):
    """Briefly changes LEDs to yellow, indicating it's too windy."""
    airports = AIRPORTS.values()
    indicator_duration = cfg.getfloat('settings', 'wind_duration', fallback=1.0)
    legend = cfg.getint('legend', 'wind', fallback=None)
    legend = [Legend('WIND', legend, FlightCategory.OFF)] if legend else []
    while True:
        # Which locations are currently breezy
        windy_airports = [airport for airport in airports if airport.windy] + legend
        log.debug('WINDY @: {}'.format(windy_airports))
        if windy_airports:
            # We want wind indicators to appear simultaneously.
            with leds.lock:
                for airport in windy_airports:
                    leds.setPixelColor(airport.index, YELLOW)
                leds.show()
                time.sleep(indicator_duration)

                for airport in windy_airports:
                    leds.setPixelColor(airport.index, airport.category.value)
                leds.show()

            time.sleep(WIND_DISPLAY_RATE - indicator_duration)
        else:
            event.wait(cfg.getint('settings', 'metar_refresh_rate', fallback=METAR_REFRESH_RATE))
            event.clear()


def set_all(leds, color=BLACK):
    """Sets all leds to a specific color."""
    for i in range(leds.numPixels()):
        leds.setPixelColor(i, color)
    leds.show()


def load_configuration():
    cfg_files = ['/etc/rpi_metar.conf', './rpi_metar.conf']

    cfg = ConfigParser()
    cfg.read(cfg_files)

    if 'megamap' in socket.gethostname():
        cfg.set('settings', 'unknown_off', 'False')
        cfg.write(open('/etc/rpi_metar.conf', 'w'))

    max_wind_speed_kts = cfg.getint('settings', 'max_wind', fallback=MAX_WIND_SPEED_KTS)
    unknown_off = cfg.getboolean('settings', 'unknown_off', fallback=True)

    for code in cfg.options('airports'):
        index = cfg.getint('airports', code)
        AIRPORTS[code.upper()] = Airport(code, index, max_wind_speed_kts=max_wind_speed_kts, unknown_off=unknown_off)

    return cfg


def on_turn(delta):
    """Let the brightness adjustment thread be aware that it needs to do something."""
    log.debug("on turn called.")
    ENCODER_QUEUE.put(delta)
    ENCODER_EVENT.set()


def adjust_brightness(leds, cfg):
    while not ENCODER_QUEUE.empty():
        delta = ENCODER_QUEUE.get() * 5
        brightness = leds.getBrightness()
        try:
            leds.setBrightness(brightness + delta)
        except OverflowError:
            log.info('New brightness exceeds limits: {}'.format(brightness + delta))
        else:
            leds.show()
            log.info('Set brightness to {}'.format(brightness + delta))

    # Now that we've handled everything in the queue, write out the current brightness into the
    # config file. This way it persists upon reboots / restarts, etc.
    if 'settings' not in cfg:
        cfg['settings'] = {}
    cfg['settings']['brightness'] = str(leds.getBrightness())
    with open('/etc/rpi_metar.conf', 'w') as f:
        cfg.write(f)
    log.info('Saved new brightness ({}) to cfg file.'.format(leds.getBrightness()))

    # Indicate that we've handled the event.
    ENCODER_EVENT.clear()


def wait_for_knob(event, leds, cfg):
    while True:
        try:
            event.wait()
            adjust_brightness(leds, cfg)
        except:
            log.exception('unexpected error')


def set_legend(leds, cfg):
    """Sets a few LEDs to fixed colors, for use with a legend."""
    if not cfg.has_section('legend'):
        return

    for category in [FlightCategory.VFR, FlightCategory.IFR, FlightCategory.MVFR, FlightCategory.LIFR]:
        index = cfg.getint('legend', category.name.casefold(), fallback=None)
        if index is not None:
            leds.setPixelColor(index, category.value)
            log.debug('Legend: set %s to %s.', index, category.name)


def get_num_leds(cfg):
    """Returns the number of LEDs as defined in the configuration file.

    It takes into account that LEDs can be defined in both the 'airports' and 'legend' sections.
    """
    airport_max = max((airport.index for airport in AIRPORTS.values()))
    legend_max = 0
    if cfg.has_section('legend'):
        legend_max = max((int(v) for v in cfg['legend'].values()))

    return max([airport_max, legend_max]) + 1


def main():

    # Register the encoder to handle changing the brightness
    knob = encoder.RotaryEncoder(callback=on_turn)

    def on_exit(sig, frame):
        knob.destroy()
        set_all(leds, BLACK)
        sys.exit(0)

    signal.signal(signal.SIGINT, on_exit)
    signal.signal(signal.SIGTERM, on_exit)

    cron.set_upgrade_schedule()

    cfg = load_configuration()

    kwargs = {
        'num': get_num_leds(cfg),
        'pin': 18,
        'gamma': GAMMA,
        'brightness': int(cfg.get('settings', 'brightness', fallback=128))
    }
    # Sometimes if we use LED strips from different batches, they behave differently with the gamma
    # controls and brightness levels. Therefore we need to be able to disable the gamma controls.
    if cfg.get('settings', 'disable_gamma', fallback=False):
        kwargs.pop('gamma')

    leds = PixelStrip(**kwargs)
    leds.begin()
    leds.lock = threading.Lock()
    set_all(leds, BLACK)

    for airport in AIRPORTS.values():
        leds.setPixelColor(airport.index, YELLOW)
    set_legend(leds, cfg)
    leds.show()

    # Kick off a thread to handle adjusting the brightness
    t = threading.Thread(name='brightness', target=wait_for_knob, args=(ENCODER_EVENT, leds, cfg))
    t.start()

    # A thread to fetch metar information periodically
    t = threading.Thread(name='metar_fetcher', target=fetch_metars, args=(METAR_QUEUE, cfg))
    t.start()

    # A thread to process metar info.
    t = threading.Thread(name='metar_processor', target=process_metars, args=(METAR_QUEUE, leds))
    t.start()

    # A thread to change the LEDs when airport categories change.
    t = threading.Thread(name='render_leds', target=render_leds, args=(LED_QUEUE, leds, cfg))
    t.start()

    # A thread for lightning
    if cfg.get('settings', 'lightning', fallback=True):
        t = threading.Thread(name='lightning', target=lightning, args=(leds, METAR_EVENT, cfg))
        t.start()

    # A thread for wind
    if cfg.get('settings', 'wind', fallback=True):
        t = threading.Thread(name='wind', target=wind, args=(leds, METAR_EVENT, cfg))
        t.start()


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

try:
    from pypandoc import convert

    def read_md(f): return convert(f, 'rst')

except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")

    def read_md(f): return open(f, 'r').read()

###############################################################################

NAME = 'rpi_metar'
PACKAGES = find_packages()
CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
]

###############################################################################

setup(
    name=NAME,
    version='0.4.1',
    packages=PACKAGES,
    include_package_data=True,
    description='Visualizing METAR data on a Raspberry Pi with LEDs.',
    keywords=['METAR', 'Raspberry Pi'],
    author='Scott Sturdivant',
    author_email='scott.sturdivant@gmail.com',
    license='MIT',
    classifiers=CLASSIFIERS,
    url='https://github.com/ScottSturdivant/rpi_metar',
    long_description=read_md('README.md'),
    install_requires=open('requirements.txt', 'r').read(),
    entry_points={
        'console_scripts': [
            'rpi_metar = rpi_metar.core:main',
            'rpi_metar_init = rpi_metar.scripts.init:main',
        ],
    },
    python_requires='>=3',
    zip_safe=False,
)
